#
# Bouygtel.pm for SMS4NOTHING
#
# Copyright (C) 2001 Julien Gaulmin (julien23) <julien23@multimania.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA or look at http://www.gnu.org/copyleft/gpl.html
#
# Contributors :
#	- Julien Gaulmin (julien23) <julien23@multimania.com>
#
# Change Log :
#	- v0.1 (julien23)   : first version (works for me :-)

package Net::SMS::Bouygtel;

use strict;

use HTTP::Cookies;
use Time::localtime;
use Time::Local;
use Date::Manip;
use LWP::UserAgent;
use HTTP::Request;

use vars qw(@ATTR $COOKIE_JAR $UA);

@ATTR = qw(recipient sender message ack date);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = bless {}, $class;

	$self->_set_attr([@ATTR, 'proxy'], @_);

	return $self;
}

sub _set_attr {
	my $self = shift;
	my $permitted_attr_aref = shift;
	my %attr = ();

	if (ref($_[0]) eq 'HASH') {
		%attr = %{$_[0]};
	} else {
		%attr = @_;
	}

	while (my($key, $value) = each %attr) {
		$key =~ s/-//;
		my $pat = qr/^$key/i;

		foreach my $attr (@$permitted_attr_aref) {
			$self->$attr($value) if $attr =~ /$pat/;
		}
	}

	$self->{_sender} ||= $ENV{USER};
}

sub recipient {
	my $self = shift;
	my $old_recipient = $self->{_recipient};

	if (@_) {
		my $new_recipient = shift;

		for ($new_recipient) {
			tr/ \t//d;
			s/^\+33/0/;
			die "Bad phone number format: $_" unless /^06\d{8}$/;

			$self->{_recipient} = $_;
		}
	}

	return $old_recipient;
}

sub message {
	my $self = shift;
	my $old_message = $self->{_message};

	if (@_) {
		my $new_message = shift;
		my $length = length($new_message);

		die "Message is too long: $length char (max 160)"
			if $length > 160;

		$self->{_message} = $new_message;
	}

	return $old_message || '';
}

sub sender {
	my $self = shift;

	return ($self->{_sender} || '', @_ && ($self->{_sender} = shift))[0];
}

sub ack {
	my $self = shift;
	my $old_ack = $self->{_ack};

	if (@_) {
		my $new_ack = shift;
		my $length = length($new_ack);

		die "Bad Email address: $new_ack" unless
			$new_ack =~ /^[\w\.-]+@(?:[\w\.-]+\.)+?[a-z]{2,3}$/i;

		$self->{_ack} = $new_ack;
	}

	return $old_ack || '';
}

sub date {
	my $self = shift;

	return ($self->{_date} || 'now', @_ && ($self->{_date} = shift))[0];
}

sub debug {
	my $self = shift;

	return ($self->{_debug}, @_ && ($self->{_debug} = shift))[0];
}

sub proxy {
	my $self = shift;

	return ($self->{_proxy}, @_ && ($self->{_proxy} = shift))[0];
}

sub verbose {
	my $self = shift;

	return ($self->{_verbose}, @_ && ($self->{_verbose} = shift))[0];
}

sub _init {
	my $self = shift;

	return if $UA;

	# Create a user agent object
	unless (defined $UA) {
		$UA = new LWP::UserAgent;

		# Fake Netscape agent on Linux
		$UA->agent('Perlzilla/1.0 (GNU/Linux)');
		$UA->timeout(30);
		if ($self->proxy()) {
			$UA->proxy('http', $self->proxy())
		} else {
			$UA->env_proxy(); # Use proxy defined in ENV by default
		}

		$UA->cookie_jar($COOKIE_JAR);
	}
}

sub send {
	my $self = shift;

	$self->_init();
	$self->_set_attr(\@ATTR, @_);

	# Create a POST request to send the message
	print "Creating the POST request...\n" if defined $self->{_debug};
	my $req = new HTTP::Request;
	$req->method('POST');
	$req->uri('http://www.6sens.com/telemessages/TelemessagesValidEnvoyeText.asp');
	$req->referer('http://www.6sens.com/telemessages/TelemessagesAnonymesProcess.asp');
	$req->content_type('application/x-www-form-urlencoded');

	my %content = (
		destinataire	=> $self->recipient(),
		msg		=> $self->message(),
		indic		=> '100'
	);
	$req->content(join '&' => map { "$_=$content{$_}" } keys %content);

	# Pass request from the user agent and get a response back
	print "Sending request...\n" if defined $self->{_debug};
	my $res = $UA->request($req);

	# Check the outcome of the response
	if ($res->is_success) {
		print "Ok, your message has been sent\n"
			if defined $self->{_verbose};
		print "Content:\n", $res->content if defined $self->{_debug};
	} else {
		select(STDERR);
		print "Error while sending message!\n",
			"May be the remote server is down (timeout).\n",
			"Or may be the daily SMS quota has been reached.\n";
		if (defined $self->{_debug}) {
			print "Part:\t\tPOST";
			print "\nCode:\t\t", $res->code if defined $res->code;
			print "\nMessage:\t", $res->message
				if defined $res->message;
			print "\nHeader:\t", $res->header
				if defined $res->header;
			print "\nContent:\n", $res->content
				if defined $res->content;
			print "\n";
		}
		exit;
	}
}

1;
